#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "../IDL/shop.h"

int main(int argc, char **argv){

	//Connection status
	RPC_STATUS status;		

	//universally unique identifier of the interface
    unsigned char * pszUuid             = NULL;

	//underlying network transport protocol
    unsigned char * pszProtocolSequence = (unsigned char*)"ncacn_np";

	//network address (NULL for local address)
    unsigned char * pszNetworkAddress   = NULL;

	//interface to use
    unsigned char * pszEndpoint			= (unsigned char*)"\\pipe\\bookshop";

	//null-terminated string representation of network options
    unsigned char * pszOptions          = NULL;

	//represent the binding object (protocol, network address, endpoint)
    unsigned char * pszStringBinding    = NULL;
    
    //exception code 
	unsigned long ulCode;

	//Book title to search
	unsigned char* title = (unsigned char*)"Divina Commedia";

	//Book found
	struct Book bookToBuy;
	
	//receipt released
	struct Receipt receipt;
 
	//creates a string binding handle

    status = RpcStringBindingCompose(pszUuid,pszProtocolSequence,
                                     pszNetworkAddress,pszEndpoint,
                                     pszOptions,&pszStringBinding);
    if (status){
        exit(status);
    }

	//returns a binding handle from a string representation of a binding object
    status = RpcBindingFromStringBinding(pszStringBinding,
                                         &server_Handle); //server binding handle
     if (status){
        exit(status);
    }
 
	//macros that control exceptions that occur outside the application code
    RpcTryExcept  
    {
		printf("Looking for book: %s \n",title);
        bookToBuy = search(title);
		printf("Book found : \n");
		printf("\t Title: %s \n", bookToBuy.name);
		printf("\t ISBN: %li \n",bookToBuy.isbn);

		receipt = order(bookToBuy);
		printf("\n\nReceipt received : \n");
		printf("\t Bank: %s \n", receipt.bank);
		printf("\t Account Number: %li \n",receipt.accountnumber);
		printf("\t Amount: %i \n",receipt.amount);
    }
    RpcExcept(1) //if some exceptions occurs
    {
        ulCode = RpcExceptionCode(); //retrives a code that identifies the type of exception that occurred
        printf("Runtime reported exception 0x%lx = %ld\n", ulCode, ulCode); 
    }
    RpcEndExcept

	//free the memory that was allocated for the string binding
    status = RpcStringFree(&pszStringBinding); 
 
    if (status) 
    {
        exit(status);
    }
	
	//release the handle to the server
    status = RpcBindingFree(&server_Handle);
 
    if (status) 
    {
        exit(status);
    }
 
    exit(0);
} 

//Allocate memory before procedure calls
void __RPC_FAR * __RPC_USER MIDL_user_allocate(size_t len)
{
    return(malloc(len));
}
 
//free memory after procedure calls
void __RPC_USER MIDL_user_free(void __RPC_FAR * ptr)
{
    free(ptr);
}